@@ -1,11 +1,11 @@
 package SVN::Notify;
 
-# $Id: Notify.pm 3084 2006-08-01 02:13:02Z theory $
+# $Id: Notify.pm 3301 2007-05-24 18:45:37Z theory $
 
 use strict;
 use constant WIN32  => $^O eq 'MSWin32';
 use constant PERL58 => $] > 5.007;
-$SVN::Notify::VERSION = '2.63';
+$SVN::Notify::VERSION = '2.66';
 
 =begin comment
 
@@ -61,7 +61,7 @@ everything executes properly.
 
 =head2 Windows Usage
 
-Go get SVN::Notify to work properly in a F<post-commit> script, you must set
+To get SVN::Notify to work properly in a F<post-commit> script, you must set
 the following environment variables, as they will likley not be present inside
 Apache:
 
@@ -76,7 +76,10 @@ Apache:
 =back
 
 See L<http://svn.haxx.se/users/archive-2006-05/0593.shtml> for more detailed
-information on getting SVN::NOtify running on Windows.
+information on getting SVN::Notify running on Windows. If you have issues with
+asynchronous execution, try using F<HookStart.exe>
+(L<http://www.koders.com/csharp/fidE2724F44EF2D47F1C0FE76C538006435FA20051D.aspx>)
+to run F<svnnotify>.
 
 =cut
 
@@ -140,7 +143,7 @@ F<post-commit> for details. Required.
 
 The address or addresses to which to send the notification email. Can be used
 multiple times to specify multiple addresses. This parameter is required
-unless C<to_regex_map> is specified.
+unless either C<to_regex_map> or C<to_email_map> is specified.
 
 =item to_regex_map
 
@@ -154,7 +157,7 @@ only if the name of one or more of the directories affected by a commit
 matches the regular expression. This is a good way to have a notification
 email sent to a particular mail address (or comma-delimited list of addresses)
 only for certain parts of the subversion tree. This parameter is required
-unless C<to> is specified.
+unless C<to> or C<to_email_map> is specified.
 
 The command-line options, C<--to-regex_map> and C<-x>, can be specified any
 number of times, once for each entry in the hash to be passed to C<new()>. The
@@ -162,6 +165,25 @@ value passed to the option must be in the form of the key and the value
 separated by an equal sign. Consult the L<Getopt::Long> documentation for more
 information.
 
+Here's an example complements of Matt Doar of how to use C<to_regex_map> to do
+per-branch matching:
+
+  author=`svnlook author $REPOS -r $REV`
+
+  # The mail regexes should match all the top-level directories
+  /usr/bin/svnnotify --repos-path "$REPOS" --revision "$REV" \
+  -x eng-bar@example.com,${EXTRAS}="^Bar" \
+  -x eng-foo@example.com,${EXTRAS}="^trunk/Foo|^branches/Foo|^tags/Foo" \
+  -x $author@example.com="^users" --subject-cx
+
+=item to_email_map
+
+  svnnotify --to-email-map L18N=translate@example.com \
+            --to-email-map License=legal@example.com
+
+The inverse of C<to_regex_map>: The regular expression is the hash key
+and the email address or addresses are the value.
+
 =item from
 
   svnnotify --from somewhere@example.com
@@ -281,9 +303,10 @@ The Perl IO layer to use for inputting and outputting data. See
 L<perlio|perlio> for details. Defaults to "encoding($charset)". If your
 repository uses different character encodings, C<charset> should be set to
 whatever is the most common character encoding, and C<io_layer> is best set to
-C<raw>. In that case, some characters might not look right in the commit
-messaage (because an email can manage only one character encoding at a time),
-but then C<svnnotify> won't get stuck issuing a slew of warnings.
+C<raw>, e.g., C<--io-layer raw --charset cp850>. In that case, some characters
+might not look right in the commit messaage (because an email can manage only
+one character encoding at a time), but then C<svnnotify> won't get stuck
+issuing a slew of warnings.
 
 =item language
 
@@ -350,10 +373,12 @@ beginning of the email body.
 =item subject_prefix
 
   svnnotify --subject-prefix [Devlist]
-  svnnotify -P (Our-Developers)
+  svnnotify -P [%d (Our-Developers)]
 
 An optional string to prepend to the beginning of the subject line of the
-notification email.
+notification email. If it contains '%d', it will be used to place the revision
+number; otherwise it will simply be prepended to the subject, which will
+contain the revision number in brackets.
 
 =item subject_cx
 
@@ -472,65 +497,92 @@ Deprecated. Use C<revision_url> instead.
 
 Deprecated. Use C<revision_url> instead.
 
+=item ticket_map
+
+  svnnotify --ticket-map '\[?#\s*(\d+)\s*\]?=http://example.com/ticket?id=%s' \
+            --ticket-map 'rt=http://rt.cpan.org/NoAuth/Bugs.html?id=%s' \
+            --ticket-map '\b([A-Z]+-\d+)\b=http://jira/browse/%s'
+
+Specifies a mapping between a regular expression and a URL. The regular
+expression should return a single match to be interpolated into the URL, which
+should be a C<sprintf> format using "%s" to place the match (usually the
+ticket identifier) from the regex. The command-line option may be specified
+any number of times for different ticketing systems. To the API, it must be
+passed as a hash reference.
+
+The first example matches "[#1234]" or "#1234" or "[# 1234]". This regex
+should be as specific as possible, preferably wrapped in "\b" to match word
+boundaries. If you're using L<SVN::Notify::HTML|SVN::Notify::HTML>, be sure to
+read its documentation for a different regular expression requirement!
+
+Optionally, the key value can be a placeholder for a regular expession used
+internally by SVN::Notify to match strings typically used for well-known
+ticketing systems. Those keys are:
+
+=over
+
+=item rt
+
+Matches Request Tracker (RT) ticket references of the form "Ticket # 12",
+"ticket 6", "RT # 52", "rt 52", "RT-Ticket # 213" or even "Ticket#1066".
+
+=item bugzilla
+
+Matches Bugzilla bug references of the form "Bug # 12" or "bug 6" or even
+"Bug#1066".
+
+=item jira
+
+Matches JIRA references of the form "JRA-1234".
+
+=item gnats
+
+Matches GnatsWeb references of the form "PR 1234".
+
+=back
+
 =item rt_url
 
   svnnotify --rt-url 'http://rt.cpan.org/NoAuth/Bugs.html?id=%s'
   svnnotify -T 'http://rt.perl.org/NoAuth/Bugs.html?id=%s'
 
-The URL of a Request Tracker (RT) server. If passed in, any strings in the log
-message of the form "Ticket # 12" or "ticket 6" or even "Ticket#1066" will be
-turned into links to the RT server. The URL must have the "%s" format where
-the RT ticket ID should be put into the URL.
+A shortcut for C<--ticket-map 'rt=$url'> provided for backwards compatibility.
 
 =item bugzilla_url
 
   svnnotify --bugzilla-url 'http://bugzilla.mozilla.org/show_bug.cgi?id=%s'
   svnnotify -B 'http://bugs.bricolage.cc/show_bug.cgi?id=%s'
 
-The URL of a Bugzilla server. If passed in, any strings in the log message of
-the form "Bug # 12" or "bug 6" or even "Bug#1066" will be turned into links to
-the Bugzilla server. The URL must have the "%s" format where the Bugzilla Bug
-ID should be put into the URL.
+A shortcut for C<--ticket-map 'bugzilla=$url'> provided for backwards
+compatibility.
 
 =item jira_url
 
   svnnotify --jira-url 'http://jira.atlassian.com/secure/ViewIssue.jspa?key=%s'
   svnnotify -J 'http://nagoya.apache.org/jira/secure/ViewIssue.jspa?key=%s'
 
-The URL of a JIRA server. If passed in, any strings in the log message that
-appear to be JIRA keys (such as "JRA-1234") will be turned into links to the
-JIRA server. The URL must have the "%s" format where the Jira key should be
-put into the URL.
+A shortcut for C<--ticket-map 'jira=$url'> provided for backwards
+compatibility.
 
 =item gnats_url
 
   svnnotify --gnats-url 'http://gnatsweb.example.com/cgi-bin/gnatsweb.pl?cmd=view&pr=%s'
   svnnotify -G 'http://gnatsweb.example.com/cgi-bin/gnatsweb.pl?cmd=view&pr=%s'
 
-The URL of a GnatsWeb server. If passed in, any strings in the log message
-that appear to be GNATS PRs (such as "PR 1234") will be turned into links to
-the GnatsWeb server. The URL must have the "%s" format where the GNATS PR
-number should be put into the URL.
+A shortcut for C<--ticket-map 'gnats=$url'> provided for backwards
+compatibility.
 
 =item ticket_url
 
   svnnotify --ticket-url 'http://ticket.example.com/showticket.html?id=%s'
 
-The URL of a custom ticket system. If passed in, any strings in the log
-message that match C<ticket_regex> will be turned into links to the custom
-ticket system. The URL must have the "%s" format where the first match
-(usually the ticket identifier) in the regex should be put into the URL.
+Deprecated. Use C<ticket_map>, instead.
 
 =item ticket_regex
 
   svnnotify --ticket-regex '\[?#\s*(\d+)\s*\]?'
 
-The regex to match a ticket tag of a custom ticket system. This should return
-a single match to be interpolated into the C<ticket_url> option. The example
-shown matches "[#1234]" or "#1234" or "[# 1234]". This regex should be as
-specific as possible, preferably wrapped in "\b" to match word boundaries. If
-you're using L<SVN::Notify::HTML|SVN::Notify::HTML>, be sure to read its
-documentation for a different syntax for C<ticket_regex>!
+Deprecated. Use C<ticket_map>, instead.
 
 =item verbose
 
@@ -541,7 +593,7 @@ default is 0, meaning that SVN::Notify will be silent. A value of 1 causes
 SVN::Notify to output some information about what it's doing, while 2 and 3
 each cause greater verbosity. To set the verbosity on the command line, simply
 pass the C<--verbose> or C<-V> option once for each level of verbosity, up to
-three times.
+three times. Output from SVN::Notify is sent to C<STDOUT>.
 
 =item boundary
 
@@ -557,6 +609,14 @@ C<prepare_subject()>.
 
 =cut
 
+# XXX Sneakily used by SVN::Notify::HTML. Change to use class methods?
+our %_ticket_regexen = (
+    rt       => '\b((?:rt|(?:rt-)?ticket:?)\s*#?\s*(\d+))\b',
+    bugzilla => '\b(bug\s*#?\s*(\d+))\b',
+    jira     => '\b([A-Z]+-\d+)\b',
+    gnats    => '\b(PR\s*(\d+))\b',
+);
+
 sub new {
     my ($class, %params) = @_;
 
@@ -579,8 +639,9 @@ sub new {
       unless $params{repos_path};
     die qq{Missing required "revision" parameter}
       unless $params{revision};
-    die qq{Missing required "to" or "to_regex_map" parameter}
-      unless @{ $params{to} } || $params{to_regex_map};
+    die qq{Missing required "to", "to_regex_map", or "to_email_map" parameter}
+        unless @{ $params{to} } || $params{to_regex_map}
+        || $params{to_email_map};
 
     # Set up default values.
     $params{svnlook}        ||= $ENV{SVNLOOK}  || $class->find_exe('svnlook');
@@ -603,6 +664,22 @@ sub new {
         $params{revision_url} .= '/revision/?rev=%s&view=rev'
     }
 
+    # Set up the issue tracking links.
+    my $track = $params{ticket_map};
+    if ($params{ticket_regex}) {
+        $track->{ delete $params{ticket_regex} } = delete $params{ticket_url};
+    }
+
+    for my $system (qw(rt bugzilla jira gnats)) {
+        my $param = $system . '_url';
+        if ($params{ $param }) {
+            $track->{ $system } = delete $params{ $param };
+            warn "--$system-url must have '%s' format\n"
+                unless $track->{ $system } =~ /%s/;
+        }
+    }
+    $params{ticket_map} = $track if $track;
+
     # Make it so!
     $class->_dbpnt( "Instantiating $class object") if $params{verbose};
     return bless \%params, $class;
@@ -691,6 +768,7 @@ sub get_options {
         'revision|r=s'        => \$opts->{revision},
         'to|t=s@'             => \$opts->{to},
         'to-regex-map|x=s%'   => \$opts->{to_regex_map},
+        'to-email-map=s%'     => \$opts->{to_email_map},
         'from|f=s'            => \$opts->{from},
         'user-domain|D=s'     => \$opts->{user_domain},
         'svnlook|l=s'         => \$opts->{svnlook},
@@ -712,12 +790,8 @@ sub get_options {
         'max-diff-length|e=i' => \$opts->{max_diff_length},
         'handler|H=s'         => \$opts->{handler},
         'author-url|A=s'      => \$opts->{author_url},
-        'rt-url|T=s'          => \$opts->{rt_url},
-        'bugzilla-url|B=s'    => \$opts->{bugzilla_url},
-        'jira-url|J=s'        => \$opts->{jira_url},
-        'gnats-url|G=s'       => \$opts->{gnats_url},
-        'ticket-url=s'        => \$opts->{ticket_url},
         'ticket-regex=s'      => \$opts->{ticket_regex},
+        'ticket-map=s%'       => \$opts->{ticket_map},
         'verbose|V+'          => \$opts->{verbose},
         'help|h'              => \$opts->{help},
         'man|m'               => \$opts->{man},
@@ -731,6 +805,8 @@ sub get_options {
             shift; push @{ $opts->{add_headers}{+shift} }, shift
         },
         'revision-url|U|svnweb-url|S|viewcvs-url=s' => \$opts->{revision_url},
+        'rt-url|T|bugzilla-url|B|jira-url|J|gnats-url|G||ticket-url=s'
+            => \$opts->{ticket_url},
     ) or return;
 
     # Load a subclass if one has been specified.
@@ -817,8 +893,8 @@ for:
 
 Only it returns after the call to C<prepare_recipients()> if there are no
 recipients (that is, as when recipients are specified solely by the
-C<to_regex_map> parameter and none of the regular expressions match any of the
-affected directories).
+C<to_regex_map> or C<to_email_map> parameter and none of the regular
+expressions match any of the affected directories).
 
 =cut
 
@@ -840,8 +916,9 @@ sub prepare {
 Collects and prepares a list of the notification recipients. The recipients
 are a combination of the value passed to the C<to> parameter as well as any
 email addresses specified as keys in the hash reference passed C<to_regex_map>
-parameter, where the regular expressions stored in the values match one or
-more of the names of the directories affected by the commit.
+parameter or values passed to the C<to_email_map> parameter, where the
+corresponding regular expressions stored in the hash matches one or more of
+the names of the directories affected by the commit.
 
 If the F<subject_cx> parameter to C<new()> has a true value,
 C<prepare_recipients()> also determines the directory name to use for the
@@ -852,21 +929,28 @@ context.
 sub prepare_recipients {
     my $self = shift;
     $self->_dbpnt( "Preparing recipients list") if $self->{verbose};
-    return $self unless $self->{to_regex_map} || $self->{subject_cx};
-    my $tos = $self->{to};
-    my $regexen = $self->{to_regex_map};
+    return $self unless $self->{to_regex_map} || $self->{subject_cx}
+        || $self->{to_email_map};
+    # Prevent duplication.
+    my $tos = $self->{to} = [ @{ $self->{to} } ];
+
+    my $regexen = $self->{to_regex_map} && $self->{to_email_map}
+        ? [ %{ $self->{to_regex_map} }, reverse %{ $self->{to_email_map } } ]
+        : $self->{to_regex_map} ? [ %{ $self->{to_regex_map} } ]
+        : $self->{to_email_map} ? [ reverse %{ $self->{to_email_map } } ]
+        :                         undef;
+
     if ($regexen) {
-        $regexen = {%$regexen};
         $self->_dbpnt( "Compiling regex_map regular expressions")
-          if $self->{verbose} > 1;
-        for (values %$regexen) {
+            if $self->{verbose} > 1;
+        for (my $i = 1; $i < @$regexen; $i += 2) {
             $self->_dbpnt( qq{Compiling "$_"}) if $self->{verbose} > 2;
             # Remove initial slash and compile.
-            s|^\^[/\\]|^|;
-            $_ = qr/$_/;
+            $regexen->[$i] =~ s|^\^[/\\]|^|;
+            $regexen->[$i] = qr/$regexen->[$i]/;
         }
     } else {
-        $regexen = {};
+        $regexen = [];
     }
 
     my $cx;
@@ -874,14 +958,15 @@ sub prepare_recipients {
                           $self->{repos_path}, '-r', $self->{revision});
 
     # Read in a list of the directories changed.
+    my %seen;
     while (<$fh>) {
         s/[\n\r\/\\]+$//;
-        while (my ($email, $rx) = each %$regexen) {
+        for (my $i = 0; $i < @$regexen; $i += 2) {
+            my ($email, $rx) = @{$regexen}[$i, $i + 1];
             # If the directory matches the regex, save the email.
             if (/$rx/) {
                 $self->_dbpnt( qq{"$_" matched $rx}) if $self->{verbose} > 2;
-                push @$tos, $email;
-                delete $regexen->{$email};
+                push @$tos, $email unless $seen{$email}++;
             }
         }
         # Grab the context if it's needed for the subject.
@@ -1007,9 +1092,16 @@ sub prepare_subject {
     $self->_dbpnt( "Preparing subject") if $self->{verbose};
 
     # Start with the optional message and revision number..
-    $self->{subject} .=
-      (defined $self->{subject_prefix} ?  "$self->{subject_prefix} " : '')
-      . "[$self->{revision}] ";
+    if ( defined $self->{subject_prefix} ) {
+        if ( index($self->{subject_prefix}, '%d') > 0 ) {
+            $self->{subject} .= sprintf
+                $self->{subject_prefix}, $self->{revision};
+        } else {
+            $self->{subject} .= $self->{subject_prefix} . "[$self->{revision}] ";
+        }
+    } else {
+        $self->{subject} .= "[$self->{revision}] ";
+    }
 
     # Add the context if there is one.
     if ($self->{cx}) {
@@ -1244,9 +1336,9 @@ sub start_body {
   $notifier->output_metadata($file_handle);
 
 This method outputs the metadata of the commit, including the revision number,
-author (user), and date of the revision. If the C<author_url>, C<viewcvs_url>,
-or C<revision_url> attributes have been set, then the appropriate URL(s) for
-the revision will also be output.
+author (user), and date of the revision. If the C<author_url> or
+C<revision_url> attributes have been set, then the appropriate URL(s) for the
+revision will also be output.
 
 =cut
 
@@ -1286,7 +1378,7 @@ sub output_log_message {
     my $msg = join "\n", @{$self->{message}};
     print $out "Log Message:\n-----------\n$msg\n";
 
-    # Make Revision links. Mutually exclusive with viewcvs.
+    # Make Revision links.
     if (my $url = $self->{revision_url}) {
         if (my @matches = $msg =~ /\b(?:rev(?:ision)?\s*#?\s*(\d+))\b/ig) {
             print $out "\nRevision Links:\n--------------\n";
@@ -1294,53 +1386,19 @@ sub output_log_message {
         }
     }
 
-    # Make Bugzilla links.
-    if (my $url = $self->bugzilla_url) {
-        if (my @matches = $msg =~ /\b(?:bug\s*#?\s*(\d+))\b/ig) {
-            print $out "\nBugzilla Links:\n--------------\n";
-            printf $out "    $url\n", $_ for @matches;
-        }
-    }
-
-    # Make RT links.
-    if (my $url = $self->rt_url) {
-        if (my @matches = $msg =~ /\b(?:(?:rt-)?ticket:?\s*#?\s*(\d+))\b/ig) {
-            print $out "\nRT Links:\n--------\n";
-            printf $out "    $url\n", $_ for @matches;
-        }
-    }
-
-    # Make JIRA links.
-    if (my $url = $self->jira_url) {
-        if (my @matches = $msg =~ /\b([A-Z]+-\d+)\b/g) {
-            print $out "\nJIRA Links:\n----------\n";
-            printf $out "    $url\n", $_ for @matches;
-        }
-    }
-
-    # Make GNATS links.
-    if (my $url = $self->gnats_url) {
-        if (my @matches = $msg =~ /\b(?:PR\s*(\d+))\b/ig) {
-            print $out "\nGNATS Links:\n-----------\n";
-            printf $out "    $url\n", $_ for @matches;
-        }
-     }
-
-    # Make custom ticketing system links.
-    if (my $url = $self->ticket_url) {
-        my $regex = $self->ticket_regex
-            or die q{Missing "ticket_regex" parameter to accompany }
-            . q{"ticket_url" parameter};
-        if (my @matches = $msg =~ /$regex/ig) {
-            print $out "\nTicket Links:\n-----------\n";
-            printf $out "    $url\n", $_ for @matches;
-        }
-    }
-
-    else {
-        die q{Missing "ticket_url" parameter to accompany }
-            . q{"ticket_regex" parameter}
-            if $self->ticket_regex;
+    # Make ticketing system links.
+    if (my $map = $self->ticket_map) {
+        my $has_header = 0;
+        $self->run_ticket_map( sub {
+            my ($regex, $url) = @_;
+            while ($msg =~ /$regex/ig) {
+                unless ($has_header) {
+                    print $out "\nTicket Links:\n:-----------\n";
+                    $has_header = 1;
+                }
+                printf $out "    $url\n",  $2 || $1;
+            }
+        } );
     }
 
     return $self;
@@ -1348,9 +1406,32 @@ sub output_log_message {
 
 ##############################################################################
 
+=head3 run_ticket_map
+
+  $notifier->run_ticket_map( \&callback, @params );
+
+Loops over the ticket systems you have defined, calling the C<$callback>
+function for each one, pasing to it the regex, url and @params specified as
+its parameters.
+
+=cut
+
+sub run_ticket_map {
+    my ($self, $callback, @params) = @_;
+
+    # Make ticketing system links.
+    my $map = $self->ticket_map or return;
+    my $has_header = 0;
+    while (my ($regex, $url) = each %$map) {
+        $regex = $_ticket_regexen{ $regex } || $regex;
+        $callback->( $regex, $url, @params );
+    }
+}
+##############################################################################
+
 =head3 output_file_lists
 
-  $notifier->output_log_message($file_handle);
+  $notifier->output_file_lists($file_handle);
 
 Outputs the lists of modified, added, and deleted files, as well as the list
 of files for which properties were changed. The labels used for each group are
@@ -1527,6 +1608,7 @@ __PACKAGE__->_accessors(qw(
     repos_path
     revision
     to_regex_map
+    to_email_map
     from
     user_domain
     svnlook
@@ -1547,12 +1629,9 @@ __PACKAGE__->_accessors(qw(
     max_diff_length
     author_url
     revision_url
-    rt_url
-    bugzilla_url
-    jira_url
-    gnats_url
     ticket_url
     ticket_regex
+    ticket_map
     header
     footer
     verbose
@@ -1585,8 +1664,25 @@ sub _accessors {
 }
 
 # Aliases for deprecated attributes.
-sub svnweb_url  { shift->revision_url(@_) }
-sub viewcvs_url { shift->revision_url(@_) }
+sub svnweb_url   { shift->revision_url(@_) }
+sub viewcvs_url  { shift->revision_url(@_) }
+
+# Deprecated ticket URL systems.
+for my $tick (qw(rt bugzilla jira gnats)) {
+    no strict 'refs';
+    *{$tick . '_url'} = sub {
+        my $self = shift;
+        my $map = $self->{ticket_map} || {};
+        return $map->{$tick} unless @_;
+        if (my $url = shift) {
+            $map->{$tick} = $url;
+        } else {
+            delete $map->{$tick};
+        }
+        $self->{ticket_map} = $map;
+        return $self;
+    };
+}
 
 for my $attr (qw(to strip_cx_regex)) {
     no strict 'refs';
@@ -1635,6 +1731,14 @@ values for the C<to> attribute.
 Gets or sets the value of the C<to_regex_map> attribute, which is a hash
 reference of email addresses mapped to regular expressions.
 
+=head3 to_email_map
+
+  my $to_email_map = $notifier->to_email_map;
+  $notifier = $notifier->to_email_map($to_email_map);
+
+Gets or sets the value of the C<to_email_map> attribute, which is a hash
+reference of regular expressions mapped to email addresses.
+
 =head3 from
 
   my $from = $notifier->from;
@@ -1970,7 +2074,7 @@ sub get_handle {
     binmode tied(*{ $smtp->tied_fh }), ":$notifier->{io_layer}"
         if SVN::Notify::PERL58;
     $smtp->mail($notifier->{from});
-    $smtp->to(@{ $notifier->{to} });
+    $smtp->to(map { split /\s*,\s*/ } @{ $notifier->{to} });
     $smtp->data;
     tie local(*SMTP), $class, $smtp;
     return *SMTP;
@@ -2031,7 +2135,7 @@ David Wheeler <david@kineticode.com>
 
 =head1 Copyright and License
 
-Copyright (c) 2004-2006 Kineticode, Inc. All Rights Reserved.
+Copyright (c) 2004-2007 Kineticode, Inc. All Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it under the
 same terms as Perl itself.
